@@ -23,6 +23,7 @@ declare(strict_types=1);
 
 namespace OCA\Talk\Chat\Parser;
 
+use OCA\Circles\CirclesManager;
 use OCA\DAV\CardDAV\PhotoCache;
 use OCA\Talk\Chat\ChatManager;
 use OCA\Talk\Exceptions\ParticipantNotFoundException;
@@ -68,11 +69,17 @@ class SystemMessage {
 	/** @var IL10N */
 	protected $l;
 
-	/** @var string[] */
+	/**
+	 * @psalm-var array<array-key, null|string>
+	 */
 	protected $displayNames = [];
 	/** @var string[] */
 	protected $groupNames = [];
 	/** @var string[] */
+	protected $circleNames = [];
+	/** @var string[] */
+	protected $circleLinks = [];
+	/** @var string[] */
 	protected $guestNames = [];
 
 	public function __construct(IUserManager $userManager,
@@ -170,8 +177,8 @@ class SystemMessage {
 			}
 		} elseif ($message === 'call_missed') {
 			[$parsedMessage, $parsedParameters, $message] = $this->parseMissedCall($room, $parameters, $currentActorId);
-		} elseif ($message === 'call_ended') {
-			[$parsedMessage, $parsedParameters] = $this->parseCall($parameters);
+		} elseif ($message === 'call_ended' || $message === 'call_ended_everyone') {
+			[$parsedMessage, $parsedParameters] = $this->parseCall($message, $parameters, $parsedParameters);
 		} elseif ($message === 'read_only_off') {
 			$parsedMessage = $this->l->t('{actor} unlocked the conversation');
 			if ($currentUserIsActor) {
@@ -303,6 +310,22 @@ class SystemMessage {
 			} elseif ($cliIsActor) {
 				$parsedMessage = $this->l->t('An administrator removed group {group}');
 			}
+		} elseif ($message === 'circle_added') {
+			$parsedParameters['circle'] = $this->getCircle($parameters['circle']);
+			$parsedMessage = $this->l->t('{actor} added circle {circle}');
+			if ($currentUserIsActor) {
+				$parsedMessage = $this->l->t('You added circle {circle}');
+			} elseif ($cliIsActor) {
+				$parsedMessage = $this->l->t('An administrator added circle {circle}');
+			}
+		} elseif ($message === 'circle_removed') {
+			$parsedParameters['circle'] = $this->getCircle($parameters['circle']);
+			$parsedMessage = $this->l->t('{actor} removed circle {circle}');
+			if ($currentUserIsActor) {
+				$parsedMessage = $this->l->t('You removed circle {circle}');
+			} elseif ($cliIsActor) {
+				$parsedMessage = $this->l->t('An administrator removed circle {circle}');
+			}
 		} elseif ($message === 'moderator_promoted') {
 			$parsedParameters['user'] = $this->getUser($parameters['user']);
 			$parsedMessage = $this->l->t('{actor} promoted {user} to moderator');
@@ -572,7 +595,19 @@ class SystemMessage {
 
 	protected function getUser(string $uid): array {
 		if (!isset($this->displayNames[$uid])) {
-			$this->displayNames[$uid] = $this->getDisplayName($uid);
+			try {
+				$this->displayNames[$uid] = $this->getDisplayName($uid);
+			} catch (ParticipantNotFoundException $e) {
+				$this->displayNames[$uid] = null;
+			}
+		}
+
+		if ($this->displayNames[$uid] === null) {
+			return [
+				'type' => 'highlight',
+				'id' => 'deleted_user',
+				'name' => $this->l->t('Deleted user'),
+			];
 		}
 
 		return [
@@ -582,6 +617,15 @@ class SystemMessage {
 		];
 	}
 
+	protected function getDisplayName(string $uid): string {
+		$user = $this->userManager->get($uid);
+		if ($user instanceof IUser) {
+			return $user->getDisplayName();
+		}
+
+		throw new ParticipantNotFoundException();
+	}
+
 	protected function getGroup(string $gid): array {
 		if (!isset($this->groupNames[$gid])) {
 			$this->groupNames[$gid] = $this->getDisplayNameGroup($gid);
@@ -594,12 +638,25 @@ class SystemMessage {
 		];
 	}
 
-	protected function getDisplayName(string $uid): string {
-		$user = $this->userManager->get($uid);
-		if ($user instanceof IUser) {
-			return $user->getDisplayName();
+	protected function getCircle(string $circleId): array {
+		if (!isset($this->circleNames[$circleId])) {
+			$this->loadCircleDetails($circleId);
 		}
-		return $uid;
+
+		if (!isset($this->circleNames[$circleId])) {
+			return [
+				'type' => 'highlight',
+				'id' => $circleId,
+				'name' => $circleId,
+			];
+		}
+
+		return [
+			'type' => 'circle',
+			'id' => $circleId,
+			'name' => $this->circleNames[$circleId],
+			'url' => $this->circleLinks[$circleId],
+		];
 	}
 
 	protected function getDisplayNameGroup(string $gid): string {
@@ -610,6 +667,20 @@ class SystemMessage {
 		return $gid;
 	}
 
+	protected function loadCircleDetails(string $circleId): void {
+		try {
+			$circlesManager = \OC::$server->get(CirclesManager::class);
+			$circlesManager->startSuperSession();
+			$circle = $circlesManager->getCircle($circleId);
+			$circlesManager->stopSession();
+
+			$this->circleNames[$circleId] = $circle->getDisplayName();
+			$this->circleLinks[$circleId] = $circle->getUrl();
+		} catch (\Exception $e) {
+			$circlesManager->stopSession();
+		}
+	}
+
 	protected function getGuest(Room $room, string $actorId): array {
 		if (!isset($this->guestNames[$actorId])) {
 			$this->guestNames[$actorId] = $this->getGuestName($room, $actorId);
@@ -663,50 +734,103 @@ class SystemMessage {
 	}
 
 
-	protected function parseCall(array $parameters): array {
+	protected function parseCall(string $message, array $parameters, array $params): array {
+		if ($message === 'call_ended_everyone') {
+			if ($params['actor']['type'] === 'user') {
+				$flipped = array_flip($parameters['users']);
+				unset($flipped[$params['actor']['id']]);
+				$parameters['users'] = array_flip($flipped);
+			} else {
+				$parameters['guests']--;
+			}
+		}
 		sort($parameters['users']);
 		$numUsers = \count($parameters['users']);
 		$displayedUsers = $numUsers;
 
 		switch ($numUsers) {
 			case 0:
-				$subject = $this->l->n(
-					'Call with %n guest (Duration {duration})',
-					'Call with %n guests (Duration {duration})',
-					$parameters['guests']
-				);
+				if ($message === 'call_ended') {
+					$subject = $this->l->n(
+						'Call with %n guest (Duration {duration})',
+						'Call with %n guests (Duration {duration})',
+						$parameters['guests']
+					);
+				} else {
+					$subject = $this->l->n(
+						'{actor} ended the call with %n guest (Duration {duration})',
+						'{actor} ended the call with %n guests (Duration {duration})',
+						$parameters['guests']
+					);
+				}
 				break;
 			case 1:
-				$subject = $this->l->t('Call with {user1} and {user2} (Duration {duration})');
+				if ($message === 'call_ended') {
+					$subject = $this->l->t('Call with {user1} and {user2} (Duration {duration})');
+				} else {
+					if ($parameters['guests'] === 0) {
+						$subject = $this->l->t('{actor} ended the call with {user1} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1} and {user2} (Duration {duration})');
+					}
+				}
 				$subject = str_replace('{user2}', $this->l->n('%n guest', '%n guests', $parameters['guests']), $subject);
 				break;
 			case 2:
 				if ($parameters['guests'] === 0) {
-					$subject = $this->l->t('Call with {user1} and {user2} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1} and {user2} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1} and {user2} (Duration {duration})');
+					}
 				} else {
-					$subject = $this->l->t('Call with {user1}, {user2} and {user3} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1}, {user2} and {user3} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1}, {user2} and {user3} (Duration {duration})');
+					}
 					$subject = str_replace('{user3}', $this->l->n('%n guest', '%n guests', $parameters['guests']), $subject);
 				}
 				break;
 			case 3:
 				if ($parameters['guests'] === 0) {
-					$subject = $this->l->t('Call with {user1}, {user2} and {user3} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1}, {user2} and {user3} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1}, {user2} and {user3} (Duration {duration})');
+					}
 				} else {
-					$subject = $this->l->t('Call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					}
 					$subject = str_replace('{user4}', $this->l->n('%n guest', '%n guests', $parameters['guests']), $subject);
 				}
 				break;
 			case 4:
 				if ($parameters['guests'] === 0) {
-					$subject = $this->l->t('Call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1}, {user2}, {user3} and {user4} (Duration {duration})');
+					}
 				} else {
-					$subject = $this->l->t('Call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+					if ($message === 'call_ended') {
+						$subject = $this->l->t('Call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+					} else {
+						$subject = $this->l->t('{actor} ended the call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+					}
 					$subject = str_replace('{user5}', $this->l->n('%n guest', '%n guests', $parameters['guests']), $subject);
 				}
 				break;
 			case 5:
 			default:
-				$subject = $this->l->t('Call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+				if ($message === 'call_ended') {
+					$subject = $this->l->t('Call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+				} else {
+					$subject = $this->l->t('{actor} ended the call with {user1}, {user2}, {user3}, {user4} and {user5} (Duration {duration})');
+				}
 				if ($numUsers === 5 && $parameters['guests'] === 0) {
 					$displayedUsers = 5;
 				} else {
@@ -716,7 +840,6 @@ class SystemMessage {
 				}
 		}
 
-		$params = [];
 		if ($displayedUsers > 0) {
 			for ($i = 1; $i <= $displayedUsers; $i++) {
 				$params['user' . $i] = $this->getUser($parameters['users'][$i - 1]);
