@@ -55,11 +55,22 @@ class Room {
 	 */
 	public const SIP_INCOMPATIBLE_REGEX = '/((\d)(?=\2+)|^0|\D)/';
 
-	public const UNKNOWN_CALL = -1;
-	public const ONE_TO_ONE_CALL = 1;
-	public const GROUP_CALL = 2;
-	public const PUBLIC_CALL = 3;
-	public const CHANGELOG_CONVERSATION = 4;
+	public const TYPE_UNKNOWN = -1;
+	public const TYPE_ONE_TO_ONE = 1;
+	public const TYPE_GROUP = 2;
+	public const TYPE_PUBLIC = 3;
+	public const TYPE_CHANGELOG = 4;
+
+	/** @deprecated Use self::TYPE_UNKNOWN */
+	public const UNKNOWN_CALL = self::TYPE_UNKNOWN;
+	/** @deprecated Use self::TYPE_ONE_TO_ONE */
+	public const ONE_TO_ONE_CALL = self::TYPE_ONE_TO_ONE;
+	/** @deprecated Use self::TYPE_GROUP */
+	public const GROUP_CALL = self::TYPE_GROUP;
+	/** @deprecated Use self::TYPE_PUBLIC */
+	public const PUBLIC_CALL = self::TYPE_PUBLIC;
+	/** @deprecated Use self::TYPE_CHANGELOG */
+	public const CHANGELOG_CONVERSATION = self::TYPE_CHANGELOG;
 
 	public const READ_WRITE = 0;
 	public const READ_ONLY = 1;
@@ -103,14 +114,18 @@ class Room {
 	public const EVENT_AFTER_LISTABLE_SET = self::class . '::postSetListable';
 	public const EVENT_BEFORE_LOBBY_STATE_SET = self::class . '::preSetLobbyState';
 	public const EVENT_AFTER_LOBBY_STATE_SET = self::class . '::postSetLobbyState';
+	public const EVENT_BEFORE_END_CALL_FOR_EVERYONE = self::class . '::preEndCallForEveryone';
+	public const EVENT_AFTER_END_CALL_FOR_EVERYONE = self::class . '::postEndCallForEveryone';
 	public const EVENT_BEFORE_SIP_ENABLED_SET = self::class . '::preSetSIPEnabled';
 	public const EVENT_AFTER_SIP_ENABLED_SET = self::class . '::postSetSIPEnabled';
+	public const EVENT_BEFORE_PERMISSIONS_SET = self::class . '::preSetPermissions';
+	public const EVENT_AFTER_PERMISSIONS_SET = self::class . '::postSetPermissions';
 	public const EVENT_BEFORE_USERS_ADD = self::class . '::preAddUsers';
 	public const EVENT_AFTER_USERS_ADD = self::class . '::postAddUsers';
 	public const EVENT_BEFORE_PARTICIPANT_TYPE_SET = self::class . '::preSetParticipantType';
 	public const EVENT_AFTER_PARTICIPANT_TYPE_SET = self::class . '::postSetParticipantType';
-	public const EVENT_BEFORE_PARTICIPANT_PUBLISHING_PERMISSIONS_SET = self::class . '::preSetParticipantPublishingPermissions';
-	public const EVENT_AFTER_PARTICIPANT_PUBLISHING_PERMISSIONS_SET = self::class . '::postSetParticipantPublishingPermissions';
+	public const EVENT_BEFORE_PARTICIPANT_PERMISSIONS_SET = self::class . '::preSetParticipantPermissions';
+	public const EVENT_AFTER_PARTICIPANT_PERMISSIONS_SET = self::class . '::postSetParticipantPermissions';
 	public const EVENT_BEFORE_USER_REMOVE = self::class . '::preRemoveUser';
 	public const EVENT_AFTER_USER_REMOVE = self::class . '::postRemoveUser';
 	public const EVENT_BEFORE_PARTICIPANT_REMOVE = self::class . '::preRemoveBySession';
@@ -169,9 +184,15 @@ class Room {
 	private $description;
 	/** @var string */
 	private $password;
+	/** @var string */
+	private $serverUrl;
 	/** @var int */
 	private $activeGuests;
 	/** @var int */
+	private $defaultPermissions;
+	/** @var int */
+	private $callPermissions;
+	/** @var int */
 	private $callFlag;
 	/** @var \DateTime|null */
 	private $activeSince;
@@ -207,7 +228,10 @@ class Room {
 								string $name,
 								string $description,
 								string $password,
+								string $serverUrl,
 								int $activeGuests,
+								int $defaultPermissions,
+								int $callPermissions,
 								int $callFlag,
 								?\DateTime $activeSince,
 								?\DateTime $lastActivity,
@@ -232,7 +256,10 @@ class Room {
 		$this->name = $name;
 		$this->description = $description;
 		$this->password = $password;
+		$this->serverUrl = $serverUrl;
 		$this->activeGuests = $activeGuests;
+		$this->defaultPermissions = $defaultPermissions;
+		$this->callPermissions = $callPermissions;
 		$this->callFlag = $callFlag;
 		$this->activeSince = $activeSince;
 		$this->lastActivity = $lastActivity;
@@ -288,7 +315,7 @@ class Room {
 	}
 
 	public function getName(): string {
-		if ($this->type === self::ONE_TO_ONE_CALL) {
+		if ($this->type === self::TYPE_ONE_TO_ONE) {
 			if ($this->name === '') {
 				// TODO use DI
 				$participantService = \OC::$server->get(ParticipantService::class);
@@ -327,6 +354,14 @@ class Room {
 		return $this->activeGuests;
 	}
 
+	public function getDefaultPermissions(): int {
+		return $this->defaultPermissions;
+	}
+
+	public function getCallPermissions(): int {
+		return $this->callPermissions;
+	}
+
 	public function getCallFlag(): int {
 		return $this->callFlag;
 	}
@@ -366,6 +401,14 @@ class Room {
 		return $this->password;
 	}
 
+	public function getServerUrl(): string {
+		return $this->serverUrl;
+	}
+
+	public function isFederatedRemoteRoom(): bool {
+		return $this->serverUrl !== '';
+	}
+
 	public function setParticipant(?string $userId, Participant $participant): void {
 		$this->currentUser = $userId;
 		$this->participant = $participant;
@@ -444,7 +487,7 @@ class Room {
 			}
 		}
 
-		$result = $query->execute();
+		$result = $query->executeQuery();
 		$row = $result->fetch();
 		$result->closeCursor();
 
@@ -479,7 +522,7 @@ class Room {
 			->where($query->expr()->eq('s.session_id', $query->createNamedParameter($sessionId)))
 			->andWhere($query->expr()->eq('a.room_id', $query->createNamedParameter($this->getId())))
 			->setMaxResults(1);
-		$result = $query->execute();
+		$result = $query->executeQuery();
 		$row = $result->fetch();
 		$result->closeCursor();
 
@@ -503,7 +546,7 @@ class Room {
 			->where($query->expr()->eq('a.pin', $query->createNamedParameter($pin)))
 			->andWhere($query->expr()->eq('a.room_id', $query->createNamedParameter($this->getId())))
 			->setMaxResults(1);
-		$result = $query->execute();
+		$result = $query->executeQuery();
 		$row = $result->fetch();
 		$result->closeCursor();
 
@@ -545,7 +588,7 @@ class Room {
 			}
 		}
 
-		$result = $query->execute();
+		$result = $query->executeQuery();
 		$row = $result->fetch();
 		$result->closeCursor();
 
@@ -593,7 +636,7 @@ class Room {
 			}
 		}
 
-		$result = $query->execute();
+		$result = $query->executeQuery();
 		$row = $result->fetch();
 		$result->closeCursor();
 
@@ -607,17 +650,17 @@ class Room {
 	public function deleteRoom(): void {
 		$event = new RoomEvent($this);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_ROOM_DELETE, $event);
-		$query = $this->db->getQueryBuilder();
+		$delete = $this->db->getQueryBuilder();
 
 		// Delete attendees
-		$query->delete('talk_attendees')
-			->where($query->expr()->eq('room_id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$delete->delete('talk_attendees')
+			->where($delete->expr()->eq('room_id', $delete->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$delete->executeStatement();
 
 		// Delete room
-		$query->delete('talk_rooms')
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$delete->delete('talk_rooms')
+			->where($delete->expr()->eq('id', $delete->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$delete->executeStatement();
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_ROOM_DELETE, $event);
 		if (class_exists(CriticalActionPerformedEvent::class)) {
@@ -629,7 +672,7 @@ class Room {
 	}
 
 	/**
-	 * @param string $newName Currently it is only allowed to rename: self::GROUP_CALL, self::PUBLIC_CALL
+	 * @param string $newName Currently it is only allowed to rename: self::TYPE_GROUP, self::TYPE_PUBLIC
 	 * @param string|null $oldName
 	 * @return bool True when the change was valid, false otherwise
 	 */
@@ -642,11 +685,11 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'name', $newName, $oldName);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_NAME_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('name', $query->createNamedParameter($newName))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('name', $update->createNamedParameter($newName))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 		$this->name = $newName;
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_NAME_SET, $event);
@@ -674,11 +717,11 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'description', $description, $oldDescription);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_DESCRIPTION_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('description', $query->createNamedParameter($description))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('description', $update->createNamedParameter($description))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 		$this->description = $description;
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_DESCRIPTION_SET, $event);
@@ -687,11 +730,11 @@ class Room {
 	}
 
 	/**
-	 * @param string $password Currently it is only allowed to have a password for Room::PUBLIC_CALL
+	 * @param string $password Currently it is only allowed to have a password for Room::TYPE_PUBLIC
 	 * @return bool True when the change was valid, false otherwise
 	 */
 	public function setPassword(string $password): bool {
-		if ($this->getType() !== self::PUBLIC_CALL) {
+		if ($this->getType() !== self::TYPE_PUBLIC) {
 			return false;
 		}
 
@@ -700,11 +743,11 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'password', $password);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_PASSWORD_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('password', $query->createNamedParameter($hash))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('password', $update->createNamedParameter($hash))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 		$this->password = $hash;
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_PASSWORD_SET, $event);
@@ -717,11 +760,11 @@ class Room {
 	 * @return bool
 	 */
 	public function setLastActivity(\DateTime $now): bool {
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('last_activity', $query->createNamedParameter($now, 'datetime'))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('last_activity', $update->createNamedParameter($now, 'datetime'))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->lastActivity = $now;
 
@@ -735,39 +778,39 @@ class Room {
 	 * @return bool
 	 */
 	public function setActiveSince(\DateTime $since, int $callFlag, bool $isGuest): bool {
-		if ($isGuest && $this->getType() === self::PUBLIC_CALL) {
-			$query = $this->db->getQueryBuilder();
-			$query->update('talk_rooms')
-				->set('active_guests', $query->createFunction($query->getColumnName('active_guests') . ' + 1'))
+		if ($isGuest && $this->getType() === self::TYPE_PUBLIC) {
+			$update = $this->db->getQueryBuilder();
+			$update->update('talk_rooms')
+				->set('active_guests', $update->createFunction($update->getColumnName('active_guests') . ' + 1'))
 				->set(
 					'call_flag',
-					$query->expr()->bitwiseOr('call_flag', $callFlag)
+					$update->expr()->bitwiseOr('call_flag', $callFlag)
 				)
-				->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-			$query->execute();
+				->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+			$update->executeStatement();
 
 			$this->activeGuests++;
 		} elseif (!$isGuest) {
-			$query = $this->db->getQueryBuilder();
-			$query->update('talk_rooms')
+			$update = $this->db->getQueryBuilder();
+			$update->update('talk_rooms')
 				->set(
 					'call_flag',
-					$query->expr()->bitwiseOr('call_flag', $callFlag)
+					$update->expr()->bitwiseOr('call_flag', $callFlag)
 				)
-				->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-			$query->execute();
+				->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+			$update->executeStatement();
 		}
 
 		if ($this->activeSince instanceof \DateTime) {
 			return false;
 		}
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('active_since', $query->createNamedParameter($since, IQueryBuilder::PARAM_DATE))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
-			->andWhere($query->expr()->isNull('active_since'));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('active_since', $update->createNamedParameter($since, IQueryBuilder::PARAM_DATE))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
+			->andWhere($update->expr()->isNull('active_since'));
+		$update->executeStatement();
 
 		$this->activeSince = $since;
 
@@ -775,46 +818,47 @@ class Room {
 	}
 
 	public function setLastMessage(IComment $message): void {
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('last_message', $query->createNamedParameter((int) $message->getId()))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('last_message', $update->createNamedParameter((int) $message->getId()))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->lastMessage = $message;
 		$this->lastMessageId = (int) $message->getId();
 	}
 
 	public function resetActiveSince(): bool {
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('active_guests', $query->createNamedParameter(0, IQueryBuilder::PARAM_INT))
-			->set('active_since', $query->createNamedParameter(null, IQueryBuilder::PARAM_DATE))
-			->set('call_flag', $query->createNamedParameter(0, IQueryBuilder::PARAM_INT))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
-			->andWhere($query->expr()->isNotNull('active_since'));
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('active_guests', $update->createNamedParameter(0, IQueryBuilder::PARAM_INT))
+			->set('active_since', $update->createNamedParameter(null, IQueryBuilder::PARAM_DATE))
+			->set('call_flag', $update->createNamedParameter(0, IQueryBuilder::PARAM_INT))
+			->set('call_permissions', $update->createNamedParameter(Attendee::PERMISSIONS_DEFAULT, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
+			->andWhere($update->expr()->isNotNull('active_since'));
 
 		$this->activeGuests = 0;
 		$this->activeSince = null;
 
-		return (bool) $query->execute();
+		return (bool) $update->executeStatement();
 	}
 
 	public function setAssignedSignalingServer(?int $signalingServer): bool {
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('assigned_hpb', $query->createNamedParameter($signalingServer))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('assigned_hpb', $update->createNamedParameter($signalingServer))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
 
 		if ($signalingServer !== null) {
-			$query->andWhere($query->expr()->isNull('assigned_hpb'));
+			$update->andWhere($update->expr()->isNull('assigned_hpb'));
 		}
 
-		return (bool) $query->execute();
+		return (bool) $update->executeStatement();
 	}
 
 	/**
-	 * @param int $newType Currently it is only allowed to change between `self::GROUP_CALL` and `self::PUBLIC_CALL`
+	 * @param int $newType Currently it is only allowed to change between `self::TYPE_GROUP` and `self::TYPE_PUBLIC`
 	 * @return bool True when the change was valid, false otherwise
 	 */
 	public function setType(int $newType, bool $allowSwitchingOneToOne = false): bool {
@@ -822,11 +866,11 @@ class Room {
 			return true;
 		}
 
-		if (!$allowSwitchingOneToOne && $this->getType() === self::ONE_TO_ONE_CALL) {
+		if (!$allowSwitchingOneToOne && $this->getType() === self::TYPE_ONE_TO_ONE) {
 			return false;
 		}
 
-		if (!in_array($newType, [self::GROUP_CALL, self::PUBLIC_CALL], true)) {
+		if (!in_array($newType, [self::TYPE_GROUP, self::TYPE_PUBLIC], true)) {
 			return false;
 		}
 
@@ -835,21 +879,21 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'type', $newType, $oldType);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_TYPE_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('type', $query->createNamedParameter($newType, IQueryBuilder::PARAM_INT))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('type', $update->createNamedParameter($newType, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->type = $newType;
 
-		if ($oldType === self::PUBLIC_CALL) {
+		if ($oldType === self::TYPE_PUBLIC) {
 			// Kick all guests and users that were not invited
-			$query = $this->db->getQueryBuilder();
-			$query->delete('talk_attendees')
-				->where($query->expr()->eq('room_id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
-				->andWhere($query->expr()->in('participant_type', $query->createNamedParameter([Participant::GUEST, Participant::GUEST_MODERATOR, Participant::USER_SELF_JOINED], IQueryBuilder::PARAM_INT_ARRAY)));
-			$query->execute();
+			$delete = $this->db->getQueryBuilder();
+			$delete->delete('talk_attendees')
+				->where($delete->expr()->eq('room_id', $delete->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)))
+				->andWhere($delete->expr()->in('participant_type', $delete->createNamedParameter([Participant::GUEST, Participant::GUEST_MODERATOR, Participant::USER_SELF_JOINED], IQueryBuilder::PARAM_INT_ARRAY)));
+			$delete->executeStatement();
 		}
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_TYPE_SET, $event);
@@ -861,7 +905,7 @@ class Room {
 	 * @param int $newState Currently it is only allowed to change between
 	 * 						`self::READ_ONLY` and `self::READ_WRITE`
 	 * 						Also it's only allowed on rooms of type
-	 * 						`self::GROUP_CALL` and `self::PUBLIC_CALL`
+	 * 						`self::TYPE_GROUP` and `self::TYPE_PUBLIC`
 	 * @return bool True when the change was valid, false otherwise
 	 */
 	public function setReadOnly(int $newState): bool {
@@ -870,7 +914,7 @@ class Room {
 			return true;
 		}
 
-		if (!in_array($this->getType(), [self::GROUP_CALL, self::PUBLIC_CALL, self::CHANGELOG_CONVERSATION], true)) {
+		if (!in_array($this->getType(), [self::TYPE_GROUP, self::TYPE_PUBLIC, self::TYPE_CHANGELOG], true)) {
 			return false;
 		}
 
@@ -881,11 +925,11 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'readOnly', $newState, $oldState);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_READONLY_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('read_only', $query->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('read_only', $update->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->readOnly = $newState;
 
@@ -897,7 +941,7 @@ class Room {
 	/**
 	 * @param int $newState New listable scope from self::LISTABLE_*
 	 * 						Also it's only allowed on rooms of type
-	 * 						`self::GROUP_CALL` and `self::PUBLIC_CALL`
+	 * 						`self::TYPE_GROUP` and `self::TYPE_PUBLIC`
 	 * @return bool True when the change was valid, false otherwise
 	 */
 	public function setListable(int $newState): bool {
@@ -906,7 +950,7 @@ class Room {
 			return true;
 		}
 
-		if (!in_array($this->getType(), [self::GROUP_CALL, self::PUBLIC_CALL], true)) {
+		if (!in_array($this->getType(), [self::TYPE_GROUP, self::TYPE_PUBLIC], true)) {
 			return false;
 		}
 
@@ -921,11 +965,11 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'listable', $newState, $oldState);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_LISTABLE_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('listable', $query->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('listable', $update->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->listable = $newState;
 
@@ -946,7 +990,7 @@ class Room {
 	public function setLobby(int $newState, ?\DateTime $dateTime, bool $timerReached = false): bool {
 		$oldState = $this->lobbyState;
 
-		if (!in_array($this->getType(), [self::GROUP_CALL, self::PUBLIC_CALL], true)) {
+		if (!in_array($this->getType(), [self::TYPE_GROUP, self::TYPE_PUBLIC], true)) {
 			return false;
 		}
 
@@ -961,12 +1005,12 @@ class Room {
 		$event = new ModifyLobbyEvent($this, 'lobby', $newState, $oldState, $dateTime, $timerReached);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_LOBBY_STATE_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('lobby_state', $query->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
-			->set('lobby_timer', $query->createNamedParameter($dateTime, IQueryBuilder::PARAM_DATE))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('lobby_state', $update->createNamedParameter($newState, IQueryBuilder::PARAM_INT))
+			->set('lobby_timer', $update->createNamedParameter($dateTime, IQueryBuilder::PARAM_DATE))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->lobbyState = $newState;
 
@@ -982,7 +1026,7 @@ class Room {
 			return false;
 		}
 
-		if (!in_array($this->getType(), [self::GROUP_CALL, self::PUBLIC_CALL], true)) {
+		if (!in_array($this->getType(), [self::TYPE_GROUP, self::TYPE_PUBLIC], true)) {
 			return false;
 		}
 
@@ -997,16 +1041,35 @@ class Room {
 		$event = new ModifyRoomEvent($this, 'sipEnabled', $newSipEnabled, $oldSipEnabled);
 		$this->dispatcher->dispatch(self::EVENT_BEFORE_SIP_ENABLED_SET, $event);
 
-		$query = $this->db->getQueryBuilder();
-		$query->update('talk_rooms')
-			->set('sip_enabled', $query->createNamedParameter($newSipEnabled, IQueryBuilder::PARAM_INT))
-			->where($query->expr()->eq('id', $query->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
-		$query->execute();
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set('sip_enabled', $update->createNamedParameter($newSipEnabled, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
 
 		$this->sipEnabled = $newSipEnabled;
 
 		$this->dispatcher->dispatch(self::EVENT_AFTER_SIP_ENABLED_SET, $event);
 
+		return true;
+	}
+
+	public function setPermissions(string $level, int $newPermissions): bool {
+		if ($level !== 'default' && $level !== 'call') {
+			return false;
+		}
+
+		$update = $this->db->getQueryBuilder();
+		$update->update('talk_rooms')
+			->set($level . '_permissions', $update->createNamedParameter($newPermissions, IQueryBuilder::PARAM_INT))
+			->where($update->expr()->eq('id', $update->createNamedParameter($this->getId(), IQueryBuilder::PARAM_INT)));
+		$update->executeStatement();
+
+		if ($level === 'default') {
+			$this->defaultPermissions = $newPermissions;
+		} else {
+			$this->callPermissions = $newPermissions;
+		}
 
 		return true;
 	}
